@@ -79,7 +79,7 @@ my class Array { # declared in BOOTSTRAP
                     nqp::ifnull(
                       nqp::atpos($!reified,$!i = nqp::add_i($!i,1)),
                       nqp::islt_i($!i,nqp::elems($!reified))
-                        ?? self!found-hole
+                        ?? self!hole($!i)
                         !! $!todo.DEFINITE
                           ?? nqp::islt_i($!i,$!todo.reify-at-least(nqp::add_i($!i,1)))
                             ?? nqp::atpos($!reified,$!i) # cannot be nqp::null
@@ -87,12 +87,12 @@ my class Array { # declared in BOOTSTRAP
                           !! IterationEnd
                     )
                 }
-                method !found-hole() {
+                method !hole(int $i) {
                    nqp::p6bindattrinvres(
                      (my \v := nqp::p6scalarfromdesc($!descriptor)),
                      Scalar,
                      '$!whence',
-                     -> { nqp::bindpos($!reified,$!i,v) }
+                     -> { nqp::bindpos($!reified,$i,v) }
                    )
                 }
                 method !done() is raw {
@@ -101,39 +101,40 @@ my class Array { # declared in BOOTSTRAP
                 }
 
                 method push-until-lazy($target) {
-                    if $!todo.DEFINITE {
-                        my int $elems = $!todo.reify-until-lazy;
+                    nqp::if(
+                      nqp::isconcrete($!todo),
+                      nqp::stmts(
+                        (my int $elems = $!todo.reify-until-lazy),
+                        (my int $i = $!i),   # lexicals faster than attributes
                         nqp::while(   # doesn't sink
-                          nqp::islt_i($!i = nqp::add_i($!i,1),$elems),
-                          $target.push(nqp::atpos($!reified,$!i))
-                        );
+                          nqp::islt_i($i = nqp::add_i($i,1),$elems),
+                          $target.push(nqp::atpos($!reified,$i))
+                        ),
                         nqp::if(
                           $!todo.fully-reified,
-                          self!done,
+                          nqp::stmts(
+                            ($!i = $i),
+                            self!done
+                          ),
                           nqp::stmts(
                             ($!i = nqp::sub_i($elems,1)),
                             Mu
                           )
                         )
-                    }
-                    else {
-                        my int $elems = nqp::elems($!reified);
+                      ),
+                      nqp::stmts(
+                        ($elems = nqp::elems($!reified)),
+                        ($i = $!i),
                         nqp::while(   # doesn't sink
-                          nqp::islt_i($!i = nqp::add_i($!i,1),$elems),
+                          nqp::islt_i($i = nqp::add_i($i,1),$elems),
                           $target.push(
-                            nqp::ifnull(
-                              nqp::atpos($!reified,$!i),
-                              nqp::p6bindattrinvres(
-                                (my \v := nqp::p6scalarfromdesc($!descriptor)),
-                                Scalar,
-                                '$!whence',
-                                -> { nqp::bindpos($!reified,$!i,v) }
-                              )
-                            )
+                            nqp::ifnull(nqp::atpos($!reified,$i),self!hole($i))
                           )
-                        );
+                        ),
+                        ($!i = $i),
                         IterationEnd
-                    }
+                      )
+                    )
                 }
 
                 method is-lazy() { $!todo.DEFINITE && $!todo.is-lazy }
@@ -179,7 +180,7 @@ my class Array { # declared in BOOTSTRAP
         )
     }
 
-    proto method new(|) { * }
+    proto method new(|) {*}
     multi method new(:$shape!) {
         nqp::if(
           nqp::defined($shape),
@@ -223,7 +224,7 @@ my class Array { # declared in BOOTSTRAP
         nqp::create(self).STORE(@values)
     }
 
-    proto method STORE(|) { * }
+    proto method STORE(|) {*}
     multi method STORE(Array:D: Iterable:D \iterable) {
         nqp::iscont(iterable)
             ?? self!STORE-ONE(iterable)
@@ -276,16 +277,21 @@ my class Array { # declared in BOOTSTRAP
             method iterator() {
                 Rakudo::Iterator.ReifiedArray(self,$!descriptor)
             }
-            multi method AT-POS(Int:D $pos) {
+
+            method !AT-POS-CONTAINER(Int:D \pos) {
+                nqp::p6bindattrinvres(
+                  (my $scalar := nqp::p6scalarfromdesc($!descriptor)),
+                  Scalar,
+                  '$!whence',
+                  -> { nqp::bindpos(
+                         nqp::getattr(self,List,'$!reified'),pos,$scalar) }
+                )
+            }
+
+            multi method AT-POS(Int:D \pos) {
                 nqp::ifnull(
-                  nqp::atpos(nqp::getattr(self,List,'$!reified'),$pos),
-                  nqp::p6bindattrinvres(
-                    (my $scalar := nqp::p6scalarfromdesc($!descriptor)),
-                    Scalar,
-                    '$!whence',
-                    -> { nqp::bindpos(
-                           nqp::getattr(self,List,'$!reified'),$pos,$scalar) }
-                  )
+                  nqp::atpos(nqp::getattr(self,List,'$!reified'),pos),
+                  self!AT-POS-CONTAINER(pos)
                 )
             }
             method default() { $!descriptor.default }
@@ -386,69 +392,54 @@ my class Array { # declared in BOOTSTRAP
 
     multi method AT-POS(Array:D: int $pos) is raw {
         nqp::if(
-          nqp::islt_i($pos,0),
-          Failure.new(X::OutOfRange.new(
-            :what($*INDEX // 'Index'),:got($pos),:range<0..^Inf>)),
-          nqp::if(
-            (my $reified := nqp::getattr(self,List,'$!reified')).DEFINITE,
-            nqp::ifnull(
-              nqp::atpos($reified,$pos),           # found it!
-              nqp::if(
-                nqp::islt_i(
-                  $pos,nqp::elems(nqp::getattr(self,List,'$!reified'))),
-                self!AT-POS-CONTAINER($pos),       # it's a hole
-                nqp::if(                           # too far out, try reifying
-                  (my $todo := nqp::getattr(self,List,'$!todo')).DEFINITE,
-                  nqp::stmts(
-                    $todo.reify-at-least(nqp::add_i($pos,1)),
-                    nqp::ifnull(
-                      nqp::atpos($reified,$pos),   # reified ok
-                      self!AT-POS-CONTAINER($pos)  # reifier didn't reach
-                    )
-                  ),
-                  self!AT-POS-CONTAINER($pos)      # create an outlander
-                )
-              )
-            ),
-            # no reified, implies no todo
-            nqp::stmts(                            # create reified
-              nqp::bindattr(self,List,'$!reified',nqp::create(IterationBuffer)),
-              self!AT-POS-CONTAINER($pos)          # create an outlander
-            )
-          )
+          nqp::isge_i($pos,0)
+            && nqp::isconcrete(nqp::getattr(self,List,'$!reified')),
+          nqp::ifnull(
+            nqp::atpos(nqp::getattr(self,List,'$!reified'),$pos),
+            self!AT-POS-SLOW($pos)
+          ),
+          self!AT-POS-SLOW($pos)
         )
     }
     # because this is a very hot path, we copied the code from the int candidate
     multi method AT-POS(Array:D: Int:D $pos) is raw {
         nqp::if(
-          nqp::islt_i($pos,0),
-          Failure.new(X::OutOfRange.new(
-            :what($*INDEX // 'Index'),:got($pos),:range<0..^Inf>)),
+          nqp::isge_i($pos,0)
+            && nqp::isconcrete(nqp::getattr(self,List,'$!reified')),
+          nqp::ifnull(
+            nqp::atpos(nqp::getattr(self,List,'$!reified'),$pos),
+            self!AT-POS-SLOW($pos)
+          ),
+          self!AT-POS-SLOW($pos)
+        )
+    }
+
+    # handle any lookup that's not simple
+    method !AT-POS-SLOW(\pos) is raw {
+        nqp::if(
+          nqp::islt_i(pos, 0),
+          self!index-oor(pos),
           nqp::if(
-            (my $reified := nqp::getattr(self,List,'$!reified')).DEFINITE,
-            nqp::ifnull(
-              nqp::atpos($reified,$pos),           # found it!
-              nqp::if(
-                nqp::islt_i(
-                  $pos,nqp::elems(nqp::getattr(self,List,'$!reified'))),
-                self!AT-POS-CONTAINER($pos),       # it's a hole
-                nqp::if(                           # too far out, try reifying
-                  (my $todo := nqp::getattr(self,List,'$!todo')).DEFINITE,
-                  nqp::stmts(
-                    $todo.reify-at-least(nqp::add_i($pos,1)),
-                    nqp::ifnull(
-                      nqp::atpos($reified,$pos),   # reified ok
-                      self!AT-POS-CONTAINER($pos)  # reifier didn't reach
-                    )
-                  ),
-                  self!AT-POS-CONTAINER($pos)      # create an outlander
-                )
+            nqp::isconcrete(my $reified := nqp::getattr(self,List,'$!reified')),
+            nqp::if(
+              nqp::islt_i(pos,nqp::elems($reified)),
+              self!AT-POS-CONTAINER(pos),        # it's a hole
+              nqp::if(                           # too far out, try reifying
+                nqp::isconcrete(my $todo := nqp::getattr(self,List,'$!todo')),
+                nqp::stmts(
+                  $todo.reify-at-least(nqp::add_i(pos,1)),
+                  nqp::ifnull(
+                    nqp::atpos($reified,pos),    # reified ok
+                    self!AT-POS-CONTAINER(pos)   # reifier didn't reach
+                  )
+                ),
+                self!AT-POS-CONTAINER(pos)       # create an outlander
               )
             ),
             # no reified, implies no todo
-            nqp::stmts(                            # create reified
+            nqp::stmts(                          # create reified
               nqp::bindattr(self,List,'$!reified',nqp::create(IterationBuffer)),
-              self!AT-POS-CONTAINER($pos)          # create an outlander
+              self!AT-POS-CONTAINER(pos)         # create an outlander
             )
           )
         )
@@ -463,88 +454,113 @@ my class Array { # declared in BOOTSTRAP
     }
 
     multi method ASSIGN-POS(Array:D: int $pos, Mu \assignee) {
+        # Fast path: index > 0, $!reified is set up, either have a container
+        # or no $!todo so can just bind there
+        my \reified := nqp::getattr(self,List,'$!reified');
         nqp::if(
-          nqp::islt_i($pos,0),
-          Failure.new(X::OutOfRange.new(
-            :what($*INDEX // 'Index'),:got($pos),:range<0..^Inf>)),
-          nqp::if(
-            (my $reified := nqp::getattr(self,List,'$!reified')).DEFINITE,
+          nqp::isge_i($pos, 0) && nqp::isconcrete(reified),
+          nqp::stmts(
+            (my \target := nqp::atpos(reified, $pos)),
             nqp::if(
-              nqp::existspos($reified,$pos),
-              (nqp::atpos($reified,$pos) = assignee),         # found it!
+              nqp::isnull(target),
               nqp::if(
-                nqp::islt_i($pos,nqp::elems($reified)),       # it's a hole
-                (nqp::bindpos($reified,$pos,
-                  nqp::p6scalarfromdesc($!descriptor)) = assignee),
-                nqp::if(
-                  (my $todo := nqp::getattr(self,List,'$!todo')).DEFINITE,
-                  nqp::stmts(                                 # can reify
-                    $todo.reify-at-least(nqp::add_i($pos,1)),
-                    nqp::if(
-                      nqp::existspos($reified,$pos),
-                      (nqp::atpos($reified,$pos) = assignee), # reified
-                      (nqp::bindpos($reified,$pos,            # outlander
-                        nqp::p6scalarfromdesc($!descriptor)) = assignee),
-                    )
-                  ),
-                  (nqp::bindpos($reified,$pos,                # outlander
-                    nqp::p6scalarfromdesc($!descriptor)) = assignee)
+                nqp::isconcrete(nqp::getattr(self, List, '$!todo')),
+                self!ASSIGN-POS-SLOW-PATH($pos, assignee),
+                nqp::assign(
+                  nqp::bindpos(reified, $pos, nqp::p6scalarfromdesc($!descriptor)),
+                  assignee
                 )
-              )
-            ),
-            nqp::stmts(                                       # new outlander
-              nqp::bindattr(self,List,'$!reified',nqp::create(IterationBuffer)),
-              (nqp::bindpos(nqp::getattr(self,List,'$!reified'),$pos,
-                nqp::p6scalarfromdesc($!descriptor)) = assignee)
+              ),
+              nqp::assign(target, assignee)
             )
-          )
+          ),
+          self!ASSIGN-POS-SLOW-PATH($pos, assignee)
         )
     }
+
     # because this is a very hot path, we copied the code from the int candidate
     multi method ASSIGN-POS(Array:D: Int:D $pos, Mu \assignee) {
+        # Fast path: index > 0, $!reified is set up, either have a container
+        # or no $!todo so can just bind there
+        my \reified := nqp::getattr(self,List,'$!reified');
+        my int $ipos = $pos;
+        nqp::if(
+          nqp::isge_i($ipos, 0) && nqp::isconcrete(reified),
+          nqp::stmts(
+            (my \target := nqp::atpos(reified, $ipos)),
+            nqp::if(
+              nqp::isnull(target),
+              nqp::if(
+                nqp::isconcrete(nqp::getattr(self, List, '$!todo')),
+                self!ASSIGN-POS-SLOW-PATH($pos, assignee),
+                nqp::assign(
+                  nqp::bindpos(reified, $ipos, nqp::p6scalarfromdesc($!descriptor)),
+                  assignee
+                )
+              ),
+              nqp::assign(target, assignee)
+            )
+          ),
+          self!ASSIGN-POS-SLOW-PATH($pos, assignee)
+        )
+    }
+
+    method !ASSIGN-POS-SLOW-PATH(Array:D: Int:D $pos, Mu \assignee) {
         nqp::if(
           nqp::islt_i($pos,0),
-          Failure.new(X::OutOfRange.new(
-            :what($*INDEX // 'Index'),:got($pos),:range<0..^Inf>)),
+          self!index-oor($pos),
           nqp::if(
-            (my $reified := nqp::getattr(self,List,'$!reified')).DEFINITE,
-            nqp::if(
-              nqp::existspos($reified,$pos),
-              (nqp::atpos($reified,$pos) = assignee),         # found it!
+            nqp::isconcrete(nqp::getattr(self,List,'$!reified')),
+            nqp::ifnull(
+              nqp::atpos(nqp::getattr(self,List,'$!reified'),$pos),
               nqp::if(
-                nqp::islt_i($pos,nqp::elems($reified)),       # it's a hole
-                (nqp::bindpos($reified,$pos,
-                  nqp::p6scalarfromdesc($!descriptor)) = assignee),
+                nqp::islt_i(                     # it's a hole
+                  $pos,
+                  nqp::elems(nqp::getattr(self,List,'$!reified'))
+                ),
+                nqp::bindpos(
+                  nqp::getattr(self,List,'$!reified'),
+                  $pos,
+                  nqp::p6scalarfromdesc($!descriptor)
+                ),
                 nqp::if(
-                  (my $todo := nqp::getattr(self,List,'$!todo')).DEFINITE,
-                  nqp::stmts(                                 # can reify
-                    $todo.reify-at-least(nqp::add_i($pos,1)),
-                    nqp::if(
-                      nqp::existspos($reified,$pos),
-                      (nqp::atpos($reified,$pos) = assignee), # reified
-                      (nqp::bindpos($reified,$pos,            # outlander
-                        nqp::p6scalarfromdesc($!descriptor)) = assignee),
+                  nqp::isconcrete(nqp::getattr(self,List,'$!todo')),
+                  nqp::stmts(                    # can reify
+                    nqp::getattr(self,List,'$!todo')
+                      .reify-at-least(nqp::add_i($pos,1)),
+                    nqp::ifnull(
+                      nqp::atpos(                # reified
+                        nqp::getattr(self,List,'$!reified'),
+                        $pos
+                      ),
+                      nqp::bindpos(              # outlander
+                        nqp::getattr(self,List,'$!reified'),
+                        $pos,
+                        nqp::p6scalarfromdesc($!descriptor)
+                      )
                     )
                   ),
-                  (nqp::bindpos($reified,$pos,                # outlander
-                    nqp::p6scalarfromdesc($!descriptor)) = assignee)
+                  nqp::bindpos(                  # outlander without todo
+                    nqp::getattr(self,List,'$!reified'),
+                    $pos,
+                    nqp::p6scalarfromdesc($!descriptor)
+                  )
                 )
               )
             ),
-            nqp::stmts(                                       # new outlander
+            nqp::bindpos(                        # new outlander
               nqp::bindattr(self,List,'$!reified',nqp::create(IterationBuffer)),
-              (nqp::bindpos(nqp::getattr(self,List,'$!reified'),$pos,
-                nqp::p6scalarfromdesc($!descriptor)) = assignee)
+              $pos,
+              nqp::p6scalarfromdesc($!descriptor)
             )
-          )
+          ) = assignee
         )
     }
 
     multi method BIND-POS(Array:D: int $pos, Mu \bindval) is raw {
         nqp::if(
           nqp::islt_i($pos,0),
-          Failure.new(X::OutOfRange.new(
-            :what($*INDEX // 'Index'),:got($pos),:range<0..^Inf>)),
+          self!index-oor($pos),
           nqp::stmts(
             nqp::if(
               nqp::getattr(self,List,'$!reified').DEFINITE,
@@ -565,8 +581,7 @@ my class Array { # declared in BOOTSTRAP
     multi method BIND-POS(Array:D: Int:D $pos, Mu \bindval) is raw {
         nqp::if(
           nqp::islt_i($pos,0),
-          Failure.new(X::OutOfRange.new(
-            :what($*INDEX // 'Index'),:got($pos),:range<0..^Inf>)),
+          self!index-oor($pos),
           nqp::stmts(
             nqp::if(
               nqp::getattr(self,List,'$!reified').DEFINITE,
@@ -587,8 +602,7 @@ my class Array { # declared in BOOTSTRAP
     multi method DELETE-POS(Array:D: int $pos) is raw {
         nqp::if(
           nqp::islt_i($pos,0),
-          Failure.new(X::OutOfRange.new(
-            :what($*INDEX // 'Index'),:got($pos),:range<0..^Inf>)),
+          self!index-oor($pos),
           nqp::if(
             (my $reified := nqp::getattr(self,List,'$!reified')).DEFINITE,
             nqp::stmts(
@@ -630,6 +644,12 @@ my class Array { # declared in BOOTSTRAP
         self.DELETE-POS(nqp::unbox_i($pos))
     }
 
+    method !index-oor($pos) {
+      Failure.new(X::OutOfRange.new(
+          :what($*INDEX // 'Index'), :got($pos), :range<0..^Inf>
+      ))
+    }
+
     # MUST have a separate Slip variant to have it slip
     multi method push(Array:D: Slip \value) {
         self.is-lazy
@@ -1188,9 +1208,6 @@ my class Array { # declared in BOOTSTRAP
              ~ ']'
         })
     }
-    multi method gist(Array:D:) {
-        self.gistseen('Array', { '[' ~ self.map({.gist}).join(' ') ~ ']' } )
-    }
     multi method WHICH(Array:D:) { self.Mu::WHICH }
 
 #=============== class Array is closed in src/core/TypedArray.pm ===============
