@@ -29,7 +29,7 @@ my role Blob[::T = uint8] does Positional[T] does Stringy is repr('VMArray') is
             ),
             nqp::sha1(self.decode("latin-1"))
           ),
-          ObjAt
+          ValueObjAt
         )
     }
 
@@ -47,31 +47,31 @@ my role Blob[::T = uint8] does Positional[T] does Stringy is repr('VMArray') is
     }
     multi method new(Blob: *@values) { self.new(@values) }
 
-    proto method allocate(|) { * }
-    multi method allocate(Blob:U: Int $elements) {
+    proto method allocate(|) {*}
+    multi method allocate(Blob:U: Int:D $elements) {
         nqp::setelems(nqp::create(self),$elements)
     }
-    multi method allocate(Blob:U: Int $elements, int $value) {
+    multi method allocate(Blob:U: Int:D $elements, int $value) {
         my int $elems = $elements;
         my $blob     := nqp::setelems(nqp::create(self),$elems);
         my int $i     = -1;
         nqp::bindpos_i($blob,$i,$value) while nqp::islt_i(++$i,$elems);
         $blob;
     }
-    multi method allocate(Blob:U: Int $elements, Int \value) {
+    multi method allocate(Blob:U: Int:D $elements, Int:D \value) {
         my int $value = value;
         self.allocate($elements,$value)
     }
-    multi method allocate(Blob:U: Int $elements, Mu $got) {
+    multi method allocate(Blob:U: Int:D $elements, Mu:D $got) {
         self!fail-typecheck('allocate',$got)
     }
-    multi method allocate(Blob:U: Int $elements, int @values) {
+    multi method allocate(Blob:U: Int:D $elements, int @values) {
         self!spread(nqp::setelems(nqp::create(self),$elements),@values)
     }
-    multi method allocate(Blob:U: Int $elements, Blob:D $blob) {
+    multi method allocate(Blob:U: Int:D $elements, Blob:D $blob) {
         self!spread(nqp::setelems(nqp::create(self),$elements),$blob)
     }
-    multi method allocate(Blob:U: Int $elements, @values) {
+    multi method allocate(Blob:U: Int:D $elements, @values) {
         self!spread(nqp::setelems(nqp::create(self),$elements),Blob.new(@values))
     }
 
@@ -115,7 +115,7 @@ my role Blob[::T = uint8] does Positional[T] does Stringy is repr('VMArray') is
     multi method Str(Blob:D:)   { X::Buf::AsStr.new(method => 'Str'  ).throw }
     multi method Stringy(Blob:D:) { X::Buf::AsStr.new(method => 'Stringy' ).throw }
 
-    proto method decode(|) { * }
+    proto method decode(|) {*}
     multi method decode(Blob:D:) {
         nqp::p6box_s(nqp::decode(self, 'utf8'))
     }
@@ -133,7 +133,7 @@ my role Blob[::T = uint8] does Positional[T] does Stringy is repr('VMArray') is
                   IterationEnd
                 )
             }
-        }.new(self))
+        }.new(self)).cache
     }
 
     multi method gist(Blob:D:) {
@@ -151,48 +151,47 @@ my role Blob[::T = uint8] does Positional[T] does Stringy is repr('VMArray') is
     }
 
     method subbuf(Blob:D: $from, $length?) {
-
-        my int $elems = nqp::elems(self);
-        X::OutOfRange.new(
-          what => "Len element to subbuf",
-          got  => $length,
-          range => "0..$elems",
-        ).fail if $length.DEFINITE && $length < 0;
-
-        my int $pos;
-        my int $todo;
-        if nqp::istype($from,Range) {
-            $from.int-bounds($pos, my int $max);
-            $todo = $max - $pos + 1;
-        }
-        else {
-            $pos = nqp::istype($from, Callable) ?? $from($elems) !! $from.Int;
-            $todo = $length.DEFINITE
-              ?? $length.Int min $elems - $pos
-              !! $elems - $pos;
-        }
-
-        X::OutOfRange.new(
-          what    => 'From argument to subbuf',
-          got     => $from.gist,
-          range   => "0..$elems",
-          comment => "use *-{abs $pos} if you want to index relative to the end",
-        ).fail if $pos < 0;
-        X::OutOfRange.new(
-          what => 'From argument to subbuf',
-          got  => $from.gist,
-          range => "0..$elems",
-        ).fail if $pos > $elems;
-
-        my $subbuf := nqp::create(self);
-        if $todo {
-            nqp::setelems($subbuf, $todo);
-            my int $i = -1;
-            --$pos;
-            nqp::bindpos_i($subbuf,$i,nqp::atpos_i(self,++$pos))
-              while nqp::islt_i(++$i,$todo);
-        }
-        $subbuf
+        nqp::stmts(
+          (my int $elems = nqp::elems(self)),
+          nqp::if(
+            $length.DEFINITE && $length < 0,
+            X::OutOfRange.new(
+              :what('Len element to subbuf'), :got($length), :range("0..$elems"),
+            ).fail,
+            nqp::stmts(
+              (my int $pos),
+              (my int $todo),
+              nqp::if(
+                nqp::istype($from,Range),
+                nqp::stmts(
+                  $from.int-bounds($pos, my int $max),
+                  ($todo = nqp::add_i(nqp::sub_i($max, $pos), 1))),
+                nqp::stmts(
+                  ($pos = nqp::istype($from, Callable) ?? $from($elems) !! $from.Int),
+                  ($todo = $length.DEFINITE ?? $length.Int min $elems - $pos !! $elems - $pos))),
+              nqp::if(
+                nqp::islt_i($pos, 0),
+                X::OutOfRange.new(
+                  :what('From argument to subbuf'), :got($from.gist), :range("0..$elems"),
+                  :comment("use *-{abs $pos} if you want to index relative to the end"),
+                ).fail,
+                nqp::if(
+                  nqp::isgt_i($pos, $elems),
+                  X::OutOfRange.new(
+                    :what('From argument to subbuf'), :got($from.gist), :range("0..$elems"),
+                  ).fail,
+                  nqp::if(
+                    nqp::isle_i($todo, 0),
+                    nqp::create(self), # we want zero elements; return empty Blob
+                    nqp::stmts(
+                      (my $subbuf := nqp::create(self)),
+                      nqp::setelems($subbuf, $todo),
+                      (my int $i = -1),
+                      --$pos,
+                      nqp::while(
+                        nqp::islt_i(++$i,$todo),
+                        nqp::bindpos_i($subbuf, $i, nqp::atpos_i(self, ++$pos))),
+                      $subbuf)))))))
     }
 
     method reverse(Blob:D:) {
@@ -248,7 +247,7 @@ my role Blob[::T = uint8] does Positional[T] does Stringy is repr('VMArray') is
         nqp::join($delim.Str,$list)
     }
 
-    proto method unpack(|) { * }
+    proto method unpack(|) {*}
     multi method unpack(Blob:D: Str:D $template) {
         nqp::isnull(nqp::getlexcaller('EXPERIMENTAL-PACK')) and X::Experimental.new(
             feature => "the 'unpack' method",
@@ -476,7 +475,7 @@ my role Buf[::T = uint8] does Blob[T] is repr('VMArray') is array_type(T) {
         nqp::islt_i(pos,0)
           ?? Failure.new(X::OutOfRange.new(
                :what($*INDEX // 'Index'),:got(pos),:range<0..^Inf>))
-          !! nqp::bindpos_i(self,\pos,assignee)
+          !! nqp::bindpos_i(self,pos,assignee)
     }
     multi method ASSIGN-POS(Buf:D: Int:D \pos, Mu \assignee) {
         my int $pos = nqp::unbox_i(pos);
@@ -495,7 +494,7 @@ my role Buf[::T = uint8] does Blob[T] is repr('VMArray') is array_type(T) {
                   IterationEnd
                 )
             }
-        }.new(self))
+        }.new(self)).cache
     }
 
     multi method pop(Buf:D:) {
@@ -509,11 +508,11 @@ my role Buf[::T = uint8] does Blob[T] is repr('VMArray') is array_type(T) {
           !! Failure.new(X::Cannot::Empty.new(:action<shift>,:what(self.^name)))
     }
 
-    method reallocate(Buf:D: Int $elements) { nqp::setelems(self,$elements) }
+    method reallocate(Buf:D: Int:D $elements) { nqp::setelems(self,$elements) }
 
     my $empty := nqp::list_i;
     multi method splice(Buf:D \SELF:) { my $buf = SELF; SELF = Buf.new; $buf }
-    multi method splice(Buf:D: Int $offset, $size = Whatever) {
+    multi method splice(Buf:D: Int:D $offset, $size = Whatever) {
         my int $remove = self!remove($offset,$size);
         my $result := $remove
           ?? self.subbuf($offset,$remove)  # until something smarter
@@ -521,22 +520,22 @@ my role Buf[::T = uint8] does Blob[T] is repr('VMArray') is array_type(T) {
         nqp::splice(self,$empty,$offset,$remove);
         $result
     }
-    multi method splice(Buf:D: Int $offset, $size, int $got) {
+    multi method splice(Buf:D: Int:D $offset, $size, int $got) {
         self!splice-native($offset,$size,$got)
     }
-    multi method splice(Buf:D: Int $offset, $size, Int $got) {
+    multi method splice(Buf:D: Int:D $offset, $size, Int:D $got) {
         self!splice-native($offset,$size,$got)
     }
-    multi method splice(Buf:D: Int $offset, $size, Mu $got) {
+    multi method splice(Buf:D: Int:D $offset, $size, Mu:D $got) {
         self!fail-typecheck('splice',$got)
     }
-    multi method splice(Buf:D: Int $offset, $size, Buf:D $buf) {
+    multi method splice(Buf:D: Int:D $offset, $size, Buf:D $buf) {
         self!splice-native($offset,$size,$buf)
     }
-    multi method splice(Buf:D: Int $offset, $size, int @values) {
+    multi method splice(Buf:D: Int:D $offset, $size, int @values) {
         self!splice-native($offset,$size,@values)
     }
-    multi method splice(Buf:D: Int $offset, $size, @values) {
+    multi method splice(Buf:D: Int:D $offset, $size, @values) {
         self!splice-native($offset,$size,
           self!push-list("splic",nqp::create(self),@values))
     }
@@ -549,7 +548,7 @@ my role Buf[::T = uint8] does Blob[T] is repr('VMArray') is array_type(T) {
             !! size.Int
     }
 
-    method !splice-native(Buf:D: Int $offset, $size, \x) {
+    method !splice-native(Buf:D: Int:D $offset, $size, \x) {
         my int $remove = self!remove($offset,$size);
         my $result := $remove
           ?? self.subbuf($offset,$remove)  # until something smarter
@@ -560,16 +559,16 @@ my role Buf[::T = uint8] does Blob[T] is repr('VMArray') is array_type(T) {
     }
 
     multi method push(Buf:D: int $got) { nqp::push_i(self,$got); self }
-    multi method push(Buf:D: Int $got) { nqp::push_i(self,$got); self }
-    multi method push(Buf:D: Mu $got) { self!fail-typecheck('push',$got) }
+    multi method push(Buf:D: Int:D $got) { nqp::push_i(self,$got); self }
+    multi method push(Buf:D: Mu:D $got) { self!fail-typecheck('push',$got) }
     multi method push(Buf:D: Blob:D $buf) {
         nqp::splice(self,$buf,nqp::elems(self),0)
     }
     multi method push(Buf:D: **@values) { self!pend(@values,'push') }
 
     multi method append(Buf:D: int $got) { nqp::push_i(self,$got); self }
-    multi method append(Buf:D: Int $got) { nqp::push_i(self,$got); self }
-    multi method append(Buf:D: Mu $got) { self!fail-typecheck('append',$got) }
+    multi method append(Buf:D: Int:D $got) { nqp::push_i(self,$got); self }
+    multi method append(Buf:D: Mu:D $got) { self!fail-typecheck('append',$got) }
     multi method append(Buf:D: Blob:D $buf) {
         nqp::splice(self,$buf,nqp::elems(self),0)
     }
@@ -580,14 +579,14 @@ my role Buf[::T = uint8] does Blob[T] is repr('VMArray') is array_type(T) {
     multi method append(Buf:D: *@values) { self!pend(@values,'append') }
 
     multi method unshift(Buf:D: int $got) { nqp::unshift_i(self,$got); self }
-    multi method unshift(Buf:D: Int $got) { nqp::unshift_i(self,$got); self }
-    multi method unshift(Buf:D: Mu $got) { self!fail-typecheck('unshift',$got) }
+    multi method unshift(Buf:D: Int:D $got) { nqp::unshift_i(self,$got); self }
+    multi method unshift(Buf:D: Mu:D $got) { self!fail-typecheck('unshift',$got) }
     multi method unshift(Buf:D: Blob:D $buf) { nqp::splice(self,$buf,0,0) }
     multi method unshift(Buf:D: **@values) { self!pend(@values,'unshift') }
 
     multi method prepend(Buf:D: int $got) { nqp::unshift_i(self,$got); self }
-    multi method prepend(Buf:D: Int $got) { nqp::unshift_i(self,$got); self }
-    multi method prepend(Buf:D: Mu $got) { self!fail-typecheck('prepend',$got) }
+    multi method prepend(Buf:D: Int:D $got) { nqp::unshift_i(self,$got); self }
+    multi method prepend(Buf:D: Mu:D $got) { self!fail-typecheck('prepend',$got) }
     multi method prepend(Buf:D: Blob:D $buf)  { nqp::splice(self,$buf,0,0)    }
     multi method prepend(Buf:D: int @values) { nqp::splice(self,@values,0,0) }
     multi method prepend(Buf:D:  @values) { self!pend(@values,'prepend') }
@@ -618,7 +617,7 @@ constant buf16 = Buf[uint16];
 constant buf32 = Buf[uint32];
 constant buf64 = Buf[uint64];
 
-proto sub pack(|) { * }
+proto sub pack(|) {*}
 multi sub pack(Str $template, *@items) {
     nqp::isnull(nqp::getlexcaller('EXPERIMENTAL-PACK')) and X::Experimental.new(
         feature => "the 'pack' function",
@@ -712,7 +711,7 @@ multi sub pack(@template, *@items) {
 
 multi sub infix:<~>(Blob:D \a) { a }
 multi sub infix:<~>(Blob:D $a, Blob:D $b) {
-    my $res := ($a.WHAT === $b.WHAT ?? $a !! Buf).new;
+    my $res := nqp::create(nqp::eqaddr($a.WHAT,$b.WHAT) ?? $a !! Buf.^pun);
     my $adc := nqp::decont($a);
     my $bdc := nqp::decont($b);
     my int $alen = nqp::elems($adc);
