@@ -780,9 +780,11 @@ my role X::Pod                 { }
 my class X::NYI is Exception {
     has $.feature;
     has $.did-you-mean;
+    has $.workaround;
     method message() {
         my $msg = "$.feature not yet implemented. Sorry.";
         $msg ~= "\nDid you mean: {$.did-you-mean.gist}?" if $.did-you-mean;
+        $msg ~= "\nWorkaround: $.workaround" if $.workaround;
         $msg
     }
 }
@@ -895,6 +897,16 @@ my class X::Trait::Scope is Exception {
 }
 my class X::Comp::Trait::Scope is X::Trait::Scope does X::Comp { };
 
+my class X::Exhausted is Exception {
+    has $.what;
+    has $.reason;
+    method message {
+        $.reason
+          ?? "Could not create another $.what because of: $.reason"
+          !! "Could not create another $.what"
+    }
+}
+
 my class X::OutOfRange is Exception {
     has $.what = 'Argument';
     has $.got = '<unknown>';
@@ -1037,6 +1049,8 @@ my class X::Undeclared::Symbols does X::Comp {
                 local => "temp (or dynamic var)",
                 new => "method call syntax",
                 foreach => "for",
+                use => '"v" prefix for pragma (e.g., "use v6;", "use v6.c;")',
+                need => '"v" prefix and "use" for pragma (e.g., "use v6;", "use v6.c;")',
             }
             $r ~= "Undeclared routine" ~ (%.unk_routines.elems == 1 ?? "" !! "s") ~ ":\n";
             for %.unk_routines.sort(*.key) {
@@ -1529,6 +1543,7 @@ my class X::Syntax::ConditionalOperator::SecondPartInvalid does X::Syntax {
 
 my class X::Syntax::Perl5Var does X::Syntax {
     has $.name;
+    has $.identifier-name;
     my %m =
       '$*'  => '^^ and $$',
       '$"'  => '.join() method',
@@ -1586,12 +1601,22 @@ my class X::Syntax::Perl5Var does X::Syntax {
       '%^H' => '$?FOO variables',
     ;
     method message() {
-        my $v = $.name ~~ m/ <[ $ @ % & ]> [ \^ <[ A..Z ]> | \W ] /;
+        my $name = $!name;
+        my $v    = $name ~~ m/ <[ $ @ % & ]> [ \^ <[ A..Z ]> | \W ] /;
+        my $sugg = %m{~$v};
+        if $!identifier-name and $name eq '$#' {
+            # Currently only `$#` var has this identifier business handling as
+            # there are two versions of it: $# (number formatting) and $#var
+            # https://metacpan.org/pod/perlvar#Deprecated-and-removed-variables
+            # Should generalize the logic if we get more of stuff like this.
+            $name ~= $!identifier-name;
+            $sugg  = '@' ~ $!identifier-name ~ '.end';
+        }
         $v
-          ?? %m{~$v}
-            ?? "Unsupported use of $v variable; in Perl 6 please use {%m{~$v}}"
-            !! "Unsupported use of $v variable"
-          !! 'Weird unrecognized variable name: ' ~ $.name;
+          ?? $sugg
+            ?? "Unsupported use of $name variable; in Perl 6 please use $sugg"
+            !! "Unsupported use of $name variable"
+          !! 'Weird unrecognized variable name: ' ~ $name;
     }
 }
 
@@ -1798,6 +1823,13 @@ my class X::Syntax::Extension::TooComplex does X::Syntax {
     }
 }
 
+my class X::Syntax::Coercer::TooComplex does X::Syntax {
+    method message() {
+        'Coercer is too complex. Only type objects, with optional type'
+        ~ " smileys, or empty parentheses, implying 'Any', are supported."
+    }
+}
+
 my class X::Syntax::Extension::SpecialForm does X::Syntax {
     has $.category;
     has $.opname;
@@ -1811,7 +1843,14 @@ my class X::Syntax::Extension::SpecialForm does X::Syntax {
 my class X::Syntax::InfixInTermPosition does X::Syntax {
     has $.infix;
     method message() {
-        "Preceding context expects a term, but found infix {$.infix.trim} instead";
+        my $infix := $!infix.trim;
+        "Preceding context expects a term, but found infix $infix instead."
+        ~ (
+            $.post && $.post.starts-with('end ')
+                ?? "\nDid you forget '=begin $.post.substr(4)' Pod marker?"
+                !! "\nDid you make a mistake in Pod syntax?"
+            if $infix eq '='
+        )
     }
 }
 
@@ -2033,16 +2072,16 @@ my class X::Str::Sprintf::Directives::Count is Exception {
 }
 
 my class X::Str::Sprintf::Directives::Unsupported is Exception {
-    has $.directive;
-    has $.sequence;
+    has str $.directive;
+    has str $.sequence;
     method message() {
         "Directive $.directive is not valid in sprintf format sequence $.sequence"
     }
 }
 
 my class X::Str::Sprintf::Directives::BadType is Exception {
-    has $.type;
-    has $.directive;
+    has str $.type;
+    has str $.directive;
     method message() {
         "Directive $.directive not applicable for type $.type"
     }
@@ -2107,7 +2146,8 @@ my class X::Cannot::Capture is Exception {
         "Cannot unpack or Capture `$!what.gist()`.\n"
           ~ "To create a Capture, add parentheses: \\(...)\n"
           ~ 'If unpacking in a signature, perhaps you needlessly used'
-          ~ ' parentheses? -> ($x) {} vs. -> $x {}';
+          ~ ' parentheses? -> ($x) {} vs. -> $x {}'
+          ~ "\nor missed `:` in signature unpacking? -> \&c:(Int) \{}";
     }
 }
 
@@ -2132,9 +2172,19 @@ my class X::ControlFlow is Exception {
     method message() { "$.illegal without $.enclosing" }
 }
 my class X::ControlFlow::Return is X::ControlFlow {
+    has Bool $.out-of-dynamic-scope;
+    submethod BUILD(Bool() :$!out-of-dynamic-scope) {}
+
     method illegal()   { 'return'  }
     method enclosing() { 'Routine' }
-    method message()   { 'Attempt to return outside of any Routine' }
+    method message()   {
+        'Attempt to return outside of ' ~ (
+            $!out-of-dynamic-scope
+              ?? 'immediatelly-enclosing Routine (i.e. `return` execution is'
+               ~ ' outside the dynamic scope of the Routine where `return` was used)'
+              !! 'any Routine'
+        )
+    }
 }
 
 my class X::Composition::NotComposable does X::Comp {
@@ -2257,7 +2307,9 @@ my class X::TypeCheck::Splice is X::TypeCheck does X::Comp {
 my class X::Assignment::RO is Exception {
     has $.value = "value";
     method message {
-        "Cannot modify an immutable {$.value.^name} ({$.value.gist})"
+        "Cannot modify an immutable {$!value.^name} ({
+            Rakudo::Internals.SHORT-GIST: $!value
+        })"
     }
     method typename { $.value.^name }
 }
@@ -2494,8 +2546,41 @@ my class X::Multi::Ambiguous is Exception {
     has @.ambiguous;
     has $.capture;
     method message {
-        join "\n",
-            "Ambiguous call to '$.dispatcher.name()'; these signatures all match:",
+        my @bits;
+        my @priors;
+        if $.capture {
+            for $.capture.list {
+                try @bits.push(.WHAT.perl);
+                @bits.push($_.^name) if $!;
+                when Failure {
+                    @priors.push(" " ~ .mess);
+                }
+            }
+            for $.capture.hash {
+                if .value ~~ Failure {
+                    @priors.push(" " ~ .value.mess);
+                }
+                if .value ~~ Bool {
+                    @bits.push(':' ~ ('!' x !.value) ~ .key);
+                }
+                else {
+                    try @bits.push(":$(.key)\($(.value.WHAT.?perl))");
+                    @bits.push(':' ~ .value.^name) if $!;
+                }
+            }
+        }
+        else {
+            @bits.push('...');
+        }
+        if @.ambiguous[0].signature.gist.contains: ': ' {
+            my $invocant = @bits.shift;
+            my $first = @bits ?? @bits.shift !! '';
+            @bits.unshift($invocant ~ ': ' ~ $first);
+        }
+        my $cap = '(' ~ @bits.join(", ") ~ ')';
+        @priors = flat "Earlier failures:\n", @priors, "\nFinal error:\n " if @priors;
+        @priors.join ~ join "\n",
+            "Ambiguous call to '$.dispatcher.name()$cap'; these signatures all match:",
             @.ambiguous.map(*.signature.perl)
     }
 }
@@ -2521,7 +2606,9 @@ my class X::Multi::NoMatch is Exception {
         my @priors;
         if $.capture {
             for $.capture.list {
-                try @bits.push($where ?? .perl !! .WHAT.perl );
+                try @bits.push(
+                    $where ?? Rakudo::Internals.SHORT-GIST($_) !! .WHAT.perl
+                );
                 @bits.push($_.^name) if $!;
                 when Failure {
                     @priors.push(" " ~ .mess);
@@ -2535,7 +2622,10 @@ my class X::Multi::NoMatch is Exception {
                     @bits.push(':' ~ ('!' x !.value) ~ .key);
                 }
                 else {
-                    try @bits.push(":$(.key)\($($where ?? .value.?perl !! .value.WHAT.?perl ))");
+                    try @bits.push(":$(.key)\($($where
+                        ?? Rakudo::Internals.SHORT-GIST: .value
+                        !! .value.WHAT.?perl
+                    ))");
                     @bits.push(':' ~ .value.^name) if $!;
                 }
             }
@@ -2627,8 +2717,8 @@ nqp::bindcurhllsym('P6EX', BEGIN nqp::hash(
       X::Assignment::RO.new(:$value).throw;
   },
   'X::ControlFlow::Return',
-  {
-      X::ControlFlow::Return.new().throw;
+  -> $out-of-dynamic-scope = False {
+      X::ControlFlow::Return.new(:$out-of-dynamic-scope).throw;
   },
   'X::NoDispatcher',
   -> $redispatcher {
@@ -2712,9 +2802,9 @@ my class X::EXPORTHOW::Conflict does X::Comp {
 my class X::UnitScope::Invalid does X::Syntax {
     has $.what;
     has $.where;
+    has Str:D $.suggestion = 'Please use the block form.';
     method message() {
-        "A unit-scoped $.what definition is not allowed $.where;\n"
-        ~ "Please use the block form."
+        "A unit-scoped $.what definition is not allowed $.where;\n$!suggestion"
     }
 }
 
@@ -2769,9 +2859,15 @@ my class X::InvalidTypeSmiley does X::Comp {
     }
 }
 
+my class X::MultipleTypeSmiley does X::Comp {
+    method message() {
+        "Multiple type smileys cannot be used";
+    }
+}
+
 my class X::Seq::Consumed is Exception {
     method message() {
-        "This Seq has already been iterated, and its values consumed\n" ~
+        "The iterator of this Seq is already in use/consumed by another Seq\n" ~
         "(you might solve this by adding .cache on usages of the Seq, or\n" ~
         "by assigning the Seq into an array)"
     }
@@ -2785,7 +2881,13 @@ my class X::Seq::NotIndexable is Exception {
 
 my class X::WheneverOutOfScope is Exception {
     method message() {
-        "Cannot have a 'whenever' block outside the scope of a 'supply' block"
+        "Cannot have a 'whenever' block outside the scope of a 'supply' or 'react' block"
+    }
+}
+
+my class X::Comp::WheneverOutOfScope does X::Comp {
+    method message() {
+        "Cannot have a 'whenever' block outside the scope of a 'supply' or 'react' block"
     }
 }
 
@@ -2881,18 +2983,7 @@ my class X::CompUnit::UnsatisfiedDependency is Exception {
 
 my class Exceptions::JSON {
     method process($ex) {
-        $*ERR.print:
-          Rakudo::Internals::JSON.to-json( $ex.^name => Hash.new(
-            (message => $ex.?message),
-            $ex.^attributes.grep(*.has_accessor).map: {
-                with .name.substr(2) -> $attr {
-                    $attr => (
-                        (.defined and not $_ ~~ Real|Positional|Associative)
-                        ?? .Str !! $_
-                    ) given $ex."$attr"()
-                }
-            }
-          ));
+        $*ERR.print: Rakudo::Internals::JSON.to-json($ex);
         False  # done processing
     }
 }
