@@ -68,15 +68,8 @@ my class IO::Handle {
       :$chomp = $!chomp,
       :$nl-in is copy = $!nl-in,
       Str:D :$nl-out is copy = $!nl-out,
-      :$buffer,
       :$out-buffer is copy,
     ) {
-        nqp::if(
-            $buffer.DEFINITE,
-            nqp::stmts(
-              ($out-buffer = $buffer),
-              DEPRECATED ':out-buffer argument to control handle buffering',
-                '2017.09.455.g.2.fba.0.ba.0.d', '2018.01'));
 
         nqp::if(
           $bin,
@@ -283,7 +276,7 @@ my class IO::Handle {
         nqp::eoffh($!PIO)
     }
 
-    method read-internal(Int $bytes) {
+    method read-internal(Int:D $bytes) {
         nqp::readfh($!PIO,buf8.new,nqp::unbox_i($bytes))
     }
 
@@ -325,7 +318,7 @@ my class IO::Handle {
         self.slurp(:$close).split: |c
     }
 
-    proto method words (|) { * }
+    proto method words (|) {*}
     multi method words(IO::Handle:D \SELF: $limit, :$close) {
         $!decoder or die X::IO::BinaryMode.new(:trying<words>);
         nqp::istype($limit,Whatever) || $limit == Inf
@@ -488,7 +481,7 @@ my class IO::Handle {
         ).new(self)
     }
 
-    proto method lines (|) { * }
+    proto method lines (|) {*}
     multi method lines(IO::Handle:D \SELF: $limit, :$close) {
         nqp::istype($limit,Whatever) || $limit == Inf
           ?? self.lines(:$close)
@@ -580,7 +573,7 @@ my class IO::Handle {
         }
     }
 
-    proto method seek(|) { * }
+    proto method seek(|) {*}
     multi method seek(IO::Handle:D: Int:D $offset, SeekType:D $whence = SeekFromBeginning) {
         my int $rewind = 0;
         if $!decoder {
@@ -644,7 +637,7 @@ my class IO::Handle {
         self.print(sprintf |c);
     }
 
-    proto method print(|) { * }
+    proto method print(|) {*}
     multi method print(IO::Handle:D: Str:D \x --> True) {
         $!decoder or die X::IO::BinaryMode.new(:trying<print>);
         self.write-internal($!encoder.encode-chars(x));
@@ -652,8 +645,9 @@ my class IO::Handle {
     multi method print(IO::Handle:D: **@list is raw --> True) { # is raw gives List, which is cheaper
         self.print(@list.join);
     }
+    multi method print(Junction:D \j) { j.THREAD: {self.print: $_} }
 
-    proto method put(|) { * }
+    proto method put(|) {*}
     multi method put(IO::Handle:D: Str:D \x --> True) {
         $!decoder or die X::IO::BinaryMode.new(:trying<put>);
         self.write-internal($!encoder.encode-chars(
@@ -662,6 +656,7 @@ my class IO::Handle {
     multi method put(IO::Handle:D: **@list is raw --> True) { # is raw gives List, which is cheaper
         self.put(@list.join);
     }
+    multi method put(Junction:D \j) { j.THREAD: {self.put: $_} }
 
     multi method say(IO::Handle:D: Str:D $x --> True) {
         $!decoder or die X::IO::BinaryMode.new(:trying<say>);
@@ -687,7 +682,7 @@ my class IO::Handle {
         self.write-internal($!encoder.encode-chars($!nl-out));
     }
 
-    proto method slurp-rest(|) { * }
+    proto method slurp-rest(|) {*}
     multi method slurp-rest(IO::Handle:D: :$bin! where *.so, :$close --> Buf:D) {
         # NOTE: THIS METHOD WILL BE DEPRECATED IN 6.d in favour of .slurp()
         # Testing of it in roast master has been removed and only kept in 6.c
@@ -743,7 +738,7 @@ my class IO::Handle {
         $!decoder.consume-all-chars()
     }
 
-    proto method spurt(|) { * }
+    proto method spurt(|) {*}
     multi method spurt(IO::Handle:D: Blob $data, :$close) {
         LEAVE self.close if $close;
         self.write-internal($data);
@@ -769,7 +764,7 @@ my class IO::Handle {
         nqp::flushfh($!PIO);
     }
 
-    proto method encoding(|) { * }
+    proto method encoding(|) {*}
     multi method encoding(IO::Handle:D:) { $!encoding // Nil }
     multi method encoding(IO::Handle:D: $new-encoding is copy) {
         with $new-encoding {
