@@ -47,7 +47,7 @@ my class Junction { # declared in BOOTSTRAP
         )
     }
 
-    proto method new(|) { * }
+    proto method new(|) {*}
     multi method new(Junction: \values, Str :$type!) {
         nqp::create(Junction)!SET-SELF($type,values)
     }
@@ -279,6 +279,21 @@ my class Junction { # declared in BOOTSTRAP
         nqp::atpos($!storage,$i).sink while nqp::islt_i(++$i,$elems);
     }
 
+    # Helper method for handling those cases where auto-threading doesn't cut it.
+    # Call the given Callable with each of the Junction values, and return a
+    # Junction with the results of the calls.
+    method THREAD(&call) {
+        my $values := nqp::getattr(self,Junction,'$!storage');
+        my int $i = -1;
+        my int $elems = nqp::elems($values);
+        my $result := nqp::setelems(nqp::list,$elems);
+        nqp::while(
+          nqp::islt_i(++$i,$elems),
+          nqp::bindpos($result,$i,call(nqp::atpos($values,$i)))
+        );
+        nqp::p6bindattrinvres(nqp::clone(self),Junction,'$!storage',$result)
+    }
+
     method AUTOTHREAD(&call, |args) {
         my Mu $positionals := nqp::getattr(nqp::decont(args),Capture,'@!list');
 
@@ -351,19 +366,19 @@ my class Junction { # declared in BOOTSTRAP
     }
 }
 
-proto sub any(|) is pure { * }
+proto sub any(|) is pure {*}
 #multi sub any(@values) { @values.any }  # this breaks S02-literals/radix.t
 multi sub any(+values) {  values.any }
 
-proto sub all(|) is pure { * }
+proto sub all(|) is pure {*}
 multi sub all(@values) { @values.all }
 multi sub all(+values) {  values.all }
 
-proto sub one(|) is pure { * }
+proto sub one(|) is pure {*}
 multi sub one(@values) { @values.one }
 multi sub one(+values) {  values.one }
 
-proto sub none(|) is pure { * }
+proto sub none(|) is pure {*}
 multi sub none(@values) { @values.none }
 multi sub none(+values) {  values.none }
 
