@@ -1009,14 +1009,14 @@ my class List does Iterable does Positional { # declared in BOOTSTRAP
     }
 
     proto method roll(|) is nodal {*}
-    multi method roll(List:D:) {
+    multi method roll() {
         self.is-lazy
           ?? Failure.new(X::Cannot::Lazy.new(:action('.roll from')))
           !! (my Int $elems = self.elems)
             ?? nqp::atpos($!reified, $elems.rand.floor)
             !! Nil
     }
-    multi method roll(List:D: Whatever) {
+    multi method roll(Whatever) {
         nqp::if(
           self.is-lazy,
           X::Cannot::Lazy.new(:action('.roll from')).throw,
@@ -1029,7 +1029,7 @@ my class List does Iterable does Positional { # declared in BOOTSTRAP
           ))
         )
     }
-    multi method roll(List:D: \number) {
+    multi method roll(\number) {
         number == Inf
           ?? self.roll(*)
           !! self.is-lazy
@@ -1057,15 +1057,6 @@ my class List does Iterable does Positional { # declared in BOOTSTRAP
                              IterationEnd
                          }
                      }
-                     method push-all($target --> IterationEnd) {
-                         nqp::while(
-                             $!todo,
-                             nqp::stmts(
-                                 ($target.push(nqp::atpos($!list,$!elems.rand.floor))),
-                                 ($!todo = $!todo - 1)
-                             )
-                         )
-                     }
                  }.new(self,number.Int))
               !! Seq.new(Rakudo::Iterator.Empty)
     }
@@ -1418,11 +1409,9 @@ my class List does Iterable does Positional { # declared in BOOTSTRAP
               nqp::stmts(
                 (my $iterator := Rakudo::Iterator.ReifiedList(self)),
                 nqp::if(
-                  nqp::istype($n,Callable),
-                  nqp::if(
-                    nqp::isgt_i((my $skip := -($n(0).Int)),0),
-                    $iterator.skip-at-least($skip)
-                  ),
+                  nqp::istype($n,Callable)
+                    && nqp::isgt_i((my $skip := -($n(0).Int)),0),
+                  $iterator.skip-at-least($skip),
                   nqp::unless(
                     nqp::istype($n,Whatever) || $n == Inf,
                     $iterator.skip-at-least(nqp::elems($!reified) - $n)
