@@ -28,9 +28,5 @@ class Perl does Systemic {
 Rakudo::Internals.REGISTER-DYNAMIC: '$*PERL', {
     PROCESS::<$PERL> := Perl.new;
 }
-multi sub postcircumfix:<{ }> (Perl $d, "compiler" )   {
-    # allow this silently, as we will catch it on accessing the Compiler object
-    $d.compiler
-}
 
 # vim: ft=perl6 expandtab sw=4
