@@ -191,14 +191,10 @@ my class Rakudo::Internals {
                 nqp::push(restore,nqp::decont(cont)),
                 nqp::if(
                   nqp::istype(cont,Array),
-                  nqp::push(restore,cont.clone),
+                  nqp::push(restore,my @ = cont),
                   nqp::if(
                     nqp::istype(cont,Hash),
-                    nqp::push(restore,
-                      nqp::p6bindattrinvres(
-                        Hash.^parameterize(Mu,Mu).new,
-                        Hash, '$!descriptor',
-                        nqp::getattr(cont, Hash, '$!descriptor')).STORE: cont),
+                    nqp::push(restore,my % = cont),
                     nqp::stmts(
                       nqp::pop(restore),  # lose the erroneously pushed value
                       X::Localizer::NoContainer.new(:$localizer).throw
@@ -272,11 +268,7 @@ my class Rakudo::Internals {
       'ascii',           'ascii',
       'iso-8859-1',      'iso-8859-1',
       'windows-1252',    'windows-1252',
-      'windows-1251',    'windows-1251',
-      # windows without dash
-      'windows1251',    'windows-1252',
-      'windows1252',    'windows-1251',
-      # utf with dash
+      # with dash
       'utf-8',           'utf8',
       'utf-16',          'utf16',
       'utf-32',          'utf32',
@@ -603,14 +595,7 @@ my class Rakudo::Internals {
     method initialize-sprintf-handler(--> Nil) {
         class SprintfHandler {
             method mine($x) { nqp::reprname($x) eq "P6opaque"; }
-
-            proto method int(|) {*}
-            multi method int(Mu:D \n) { n.Int }
-            multi method int(Mu:U \n) { n.Numeric.Int }
-
-            proto method float(|) {*}
-            multi method float(Numeric:D \n) { n }
-            multi method float(Mu \n) { n.Numeric }
+            method int($x) { $x.Int }
         }
         unless $sprintfHandlerInitialized {
             nqp::sprintfaddargumenthandler(SprintfHandler.new);
@@ -1329,23 +1314,24 @@ my class Rakudo::Internals {
         my \payload := nqp::getpayload($vmex);
         if nqp::elems(payload) == 1 {
             if nqp::existskey(payload, 'BAD_TYPE_FOR_DIRECTIVE') {
-                X::Str::Sprintf::Directives::BadType.new:
+                X::Str::Sprintf::Directives::BadType.new(
                     type      => nqp::atkey(nqp::atkey(payload, 'BAD_TYPE_FOR_DIRECTIVE'), 'TYPE'),
                     directive => nqp::atkey(nqp::atkey(payload, 'BAD_TYPE_FOR_DIRECTIVE'), 'DIRECTIVE'),
+                ).throw
             }
-            elsif nqp::existskey(payload, 'BAD_DIRECTIVE') {
-                X::Str::Sprintf::Directives::Unsupported.new:
+            if nqp::existskey(payload, 'BAD_DIRECTIVE') {
+                X::Str::Sprintf::Directives::Unsupported.new(
                     directive => nqp::atkey(nqp::atkey(payload, 'BAD_DIRECTIVE'), 'DIRECTIVE'),
                     sequence  => nqp::atkey(nqp::atkey(payload, 'BAD_DIRECTIVE'), 'SEQUENCE'),
+                ).throw
             }
-            elsif nqp::existskey(payload, 'DIRECTIVES_COUNT') {
-                X::Str::Sprintf::Directives::Count.new:
+            if nqp::existskey(payload, 'DIRECTIVES_COUNT') {
+                X::Str::Sprintf::Directives::Count.new(
                     args-have => nqp::atkey(nqp::atkey(payload, 'DIRECTIVES_COUNT'), 'ARGS_HAVE'),
                     args-used => nqp::atkey(nqp::atkey(payload, 'DIRECTIVES_COUNT'), 'ARGS_USED'),
+                ).throw
             }
-            else { exception }
         }
-        else { exception }
     }
 
 #- start of generated part of succ/pred ---------------------------------------
