@@ -120,11 +120,7 @@ my role Real does Numeric {
         $int_part == 0 && self < 0 ?? '-' ~ $r !! $r;
     }
 
-    multi method Real(Real:D:) { self }
-    multi method Real(Real:U:) {
-        self.Mu::Real; # issue a warning;
-        self.new
-    }
+    method Real(Real:D:) { self }
     method Bridge(Real:D:) { self.Num }
     method Int(Real:D:) { self.Bridge.Int }
     method Num(Real:D:) { self.Bridge.Num }
