@@ -9,10 +9,14 @@ sub infix:<=>(Mu \a, Mu \b) is raw {
 
 my class X::Does::TypeObject is Exception {
     has Mu $.type;
-    method message() { "Cannot use 'does' operator with a type object." }
+    has %.nameds;
+    method message() {
+        "Cannot use 'does' operator on a type object {$!type.^name}."
+          ~ ("\nAdditional named parameters: {%!nameds.perl}." if %!nameds)
+    }
 }
 
-proto sub infix:<does>(|) { * }
+proto sub infix:<does>(|) {*}
 multi sub infix:<does>(Mu:D \obj, Mu:U \rolish) is raw {
     # XXX Mutability check.
     my $role := rolish.HOW.archetypes.composable() ?? rolish !!
@@ -28,15 +32,19 @@ multi sub infix:<does>(Mu:D \obj, Mu:U \rolish, :$value! is raw) is raw {
     my \mixedin = obj.^mixin($role, :need-mixin-attribute);
     mixedin.BUILD_LEAST_DERIVED({ substr(mixedin.^mixin_attribute.Str,2) => $value });
 }
-multi sub infix:<does>(Mu:U \obj, Mu:U \role) is raw {
-    X::Does::TypeObject.new(type => obj).throw
+multi sub infix:<does>(Mu:U \obj, Mu:U \role, *%_) is raw {
+    X::Does::TypeObject.new(type => obj, nameds => %_).throw
 }
 multi sub infix:<does>(Mu:D \obj, **@roles) is raw {
     # XXX Mutability check.
     my \real-roles = eager @roles.map: -> \rolish {
-        rolish.HOW.archetypes.composable() ?? rolish !!
-            rolish.HOW.archetypes.composalizable() ?? rolish.HOW.composalize(rolish) !!
-            X::Mixin::NotComposable.new(:target(obj), :rolish(rolish)).throw
+        rolish.DEFINITE
+            ?? GENERATE-ROLE-FROM-VALUE(rolish)
+            !! rolish.HOW.archetypes.composable()
+                ?? rolish
+                !! rolish.HOW.archetypes.composalizable()
+                    ?? rolish.HOW.composalize(rolish)
+                    !! X::Mixin::NotComposable.new(:target(obj), :rolish(rolish)).throw
     }
     obj.^mixin(|real-roles).BUILD_LEAST_DERIVED({});
 }
@@ -51,7 +59,7 @@ multi sub infix:<cmp>(Rational:D \a, Rational:D \b) is default {
     a.isNaN || b.isNaN ?? a.Num cmp b.Num !! a <=> b
 }
 
-proto sub infix:<but>(|) is pure { * }
+proto sub infix:<but>(|) is pure {*}
 multi sub infix:<but>(Mu:D \obj, Mu:U \rolish) {
     my $role := rolish.HOW.archetypes.composable() ?? rolish !!
                 rolish.HOW.archetypes.composalizable() ?? rolish.HOW.composalize(rolish) !!
@@ -466,7 +474,7 @@ sub WHAT(Mu \x) { x.WHAT }
 sub HOW (Mu \x) { x.HOW }
 sub VAR (Mu \x) { x.VAR }
 
-proto sub infix:<...>(|) { * }
+proto sub infix:<...>(|) {*}
 multi sub infix:<...>(\a, Mu \b) { Seq.new(SEQUENCE(a, b).iterator) }
 multi sub infix:<...>(|lol) {
     my @lol := lol.list;
@@ -506,73 +514,24 @@ multi sub infix:<...>(|lol) {
     }
 }
 
-proto sub infix:<...^>(|) { * }
+proto sub infix:<...^>(|) {*}
 multi sub infix:<...^>(\a, Mu \b) { Seq.new(SEQUENCE(a, b, :exclude_end(1)).iterator) }
 
-proto sub infix:<…>(|) { * }
+proto sub infix:<…>(|) {*}
 multi sub infix:<…>(|c) { infix:<...>(|c) }
 
-proto sub infix:<…^>(|) { * }
+proto sub infix:<…^>(|) {*}
 multi sub infix:<…^>(|c) { infix:<...^>(|c) }
 
 multi sub undefine(Mu \x) is raw { x = Nil }
 multi sub undefine(Array \x) is raw { x = Empty }
 multi sub undefine(Hash \x) is raw { x = Empty }
 
-sub prefix:<temp>(\cont) is raw {
-    my $temp_restore := nqp::getlexcaller('!TEMP-RESTORE');
-    my int $i = nqp::elems($temp_restore);
-    while $i > 0 {
-        $i = $i - 2;
-        return-rw cont if nqp::atpos($temp_restore, $i) =:= cont;
-    }
-    if nqp::iscont(cont) {
-        nqp::push($temp_restore, cont);
-        nqp::push($temp_restore, nqp::decont(cont));
-    }
-    elsif nqp::istype(cont, Array) {
-        nqp::push($temp_restore, cont);
-        nqp::push($temp_restore, my @a = cont);
-    }
-    elsif nqp::istype(cont, Hash) {
-        nqp::push($temp_restore, cont);
-        nqp::push($temp_restore, my %h = cont);
-    }
-    elsif nqp::istype(cont, Failure) {
-        cont.exception.throw
-    }
-    else {
-        X::Localizer::NoContainer.new(localizer => 'temp').throw;
-    }
-    cont
+sub prefix:<temp>(Mu \cont) is raw {
+    Rakudo::Internals.TEMP-LET(nqp::getlexcaller('!TEMP-RESTORE'),cont,'temp')
 }
-
-sub prefix:<let>(\cont) is raw {
-    my $let_restore := nqp::getlexcaller('!LET-RESTORE');
-    my int $i = nqp::elems($let_restore);
-    while $i > 0 {
-        $i = $i - 2;
-        return-rw cont if nqp::atpos($let_restore, $i) =:= cont;
-    }
-    if nqp::iscont(cont) {
-        nqp::push($let_restore, cont);
-        nqp::push($let_restore, nqp::decont(cont));
-    }
-    elsif nqp::istype(cont, Array) {
-        nqp::push($let_restore, cont);
-        nqp::push($let_restore, my @a = cont);
-    }
-    elsif nqp::istype(cont, Hash) {
-        nqp::push($let_restore, cont);
-        nqp::push($let_restore, my %h = cont);
-    }
-    elsif nqp::istype(cont, Failure) {
-        cont.exception.throw
-    }
-    else {
-        X::Localizer::NoContainer.new(localizer => 'let').throw;
-    }
-    cont
+sub prefix:<let>(Mu \cont) is raw {
+    Rakudo::Internals.TEMP-LET(nqp::getlexcaller('!LET-RESTORE'),cont,'let')
 }
 
 # this implements the ::() indirect lookup
@@ -580,9 +539,9 @@ sub INDIRECT_NAME_LOOKUP($root, *@chunks) is raw {
     nqp::if(
       # Note that each part of @chunks itself can contain double colons.
       # That's why joining and re-splitting is necessary
-      nqp::elems(my $parts :=
-        nqp::split('::',my str $name = @chunks.join('::'))),
+      (my str $name = @chunks.join('::')),
       nqp::stmts(
+        (my $parts := nqp::split('::',$name)),
         (my str $first = nqp::shift($parts)),
         nqp::if( # move the sigil to the last part of the name if available
           nqp::elems($parts),
@@ -594,8 +553,7 @@ sub INDIRECT_NAME_LOOKUP($root, *@chunks) is raw {
                 || nqp::iseq_s($sigil,'%')
                 || nqp::iseq_s($sigil,'&'),
               nqp::stmts(
-                nqp::push($parts,
-                  nqp::concat($sigil,nqp::unbox_s(nqp::pop($parts)))),
+                nqp::push($parts,nqp::concat($sigil,nqp::pop($parts))),
                 ($first = nqp::substr($first,1))
               )
             ),
@@ -611,8 +569,8 @@ sub INDIRECT_NAME_LOOKUP($root, *@chunks) is raw {
         (my Mu $thing := nqp::if(
           $root.EXISTS-KEY('%REQUIRE_SYMBOLS')
             && (my $REQUIRE_SYMBOLS := $root.AT-KEY('%REQUIRE_SYMBOLS'))
-            && ($REQUIRE_SYMBOLS{$first}:exists),
-          $REQUIRE_SYMBOLS{$first},
+            && $REQUIRE_SYMBOLS.EXISTS-KEY($first),
+          $REQUIRE_SYMBOLS.AT-KEY($first),
           nqp::if(
             $root.EXISTS-KEY($first),
             $root.AT-KEY($first),
@@ -620,12 +578,13 @@ sub INDIRECT_NAME_LOOKUP($root, *@chunks) is raw {
               GLOBAL::.EXISTS-KEY($first),
               GLOBAL::.AT-KEY($first),
               nqp::if(
-		nqp::iseq_s($first, 'GLOBAL'),
-		GLOBAL,
+                nqp::iseq_s($first,'GLOBAL'),
+                GLOBAL,
                 X::NoSuchSymbol.new(symbol => $name).fail
               )
             )
-          ))),
+          )
+        )),
         nqp::while(
           nqp::elems($parts),
           nqp::if(
@@ -636,7 +595,7 @@ sub INDIRECT_NAME_LOOKUP($root, *@chunks) is raw {
         ),
         $thing
       ),
-      X::NoSuchSymbol.new(symbol => $name).fail
+      Failure.new(X::NoSuchSymbol.new(symbol => ""))
     )
 }
 
@@ -791,7 +750,7 @@ multi sub trait_mod:<is>(Routine $r, Str :$equiv!) {
         trait_mod:<is>($r, equiv => ::(nm));
         return;
     }
-    die "Routine given to equiv does not appear to be an operator";;
+    die "Routine given to equiv does not appear to be an operator";
 }
 
 multi sub trait_mod:<is>(Routine $r, Str :$tighter!) {
@@ -800,7 +759,7 @@ multi sub trait_mod:<is>(Routine $r, Str :$tighter!) {
         trait_mod:<is>($r, tighter => ::(nm));
         return;
     }
-    die "Routine given to tighter does not appear to be an operator";;
+    die "Routine given to tighter does not appear to be an operator";
 }
 
 multi sub trait_mod:<is>(Routine $r, Str :$looser!) {
@@ -809,7 +768,7 @@ multi sub trait_mod:<is>(Routine $r, Str :$looser!) {
         trait_mod:<is>($r, looser => ::(nm));
         return;
     }
-    die "Routine given to looser does not appear to be an operator";;
+    die "Routine given to looser does not appear to be an operator";
 }
 
 proto sub infix:<∘> (&?, &?) {*}
